"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var StatValue_1 = require("../../types/StatValue");
var regexp_util_1 = require("./regexp_util");
var mod_value_to_item_class_1 = require("./mod_value_to_item_class");
var per_minute_to_per_second_1 = require("./per_minute_to_per_second");
var f = {
    format: function (n) { return "" + n; },
    inverse: function (s) { return +s; },
    regexp: regexp_util_1.NUMBER
};
/*
 * rule of thumb: is the formatter self explanatory and only need
 * arguments => declare in this file. Otherwise create additional
 * files. `mod_value_to_item_class` needs auxiliary item_classes and
 * per_minute_to_per_second has some extra documentation
 */
exports.formatters = {
    deciseconds_to_seconds: {
        format: function (n) { return "" + n * 10; },
        inverse: function (s) { return +s / 10; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_two_0dp: {
        format: function (n) { return (n / 2).toFixed(0); },
        inverse: function (s) { return +s * 2; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_ten_0dp: {
        format: function (n) { return (n / 10).toFixed(0); },
        inverse: function (s) { return +s * 10; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_fifteen_0dp: {
        format: function (n) { return (n / 15).toFixed(0); },
        inverse: function (s) { return +s * 15; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_twenty_then_double_0dp: {
        format: function (n) { return "" + Math.floor(n / 20) * 2; },
        inverse: function (s) { return +s * 10; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_one_hundred: {
        format: function (n) { return "" + n / 100; },
        inverse: function (s) { return +s * 100; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,2}"
    },
    divide_by_one_hundred_2dp: {
        format: function (n) { return (n / 100).toFixed(2); },
        inverse: function (s) { return +s * 100; },
        regexp: regexp_util_1.NUMBER + "\\.\\d{2}"
    },
    per_minute_to_per_second: per_minute_to_per_second_1.default,
    milliseconds_to_seconds: {
        format: function (n) { return "" + n / 1000; },
        inverse: function (s) { return +s * 1000; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,3}"
    },
    negate: {
        format: function (n) { return "" + -n; },
        inverse: function (s) { return -s; },
        regexp: regexp_util_1.NUMBER
    },
    divide_by_one_hundred_and_negate: {
        format: function (n) { return "" + -n / 100; },
        inverse: function (s) { return -s * 100; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,2}"
    },
    old_leech_percent: {
        format: function (n) { return "" + n / 5; },
        inverse: function (s) { return +s * 5; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,2}"
    },
    old_leech_permyriad: {
        format: function (n) { return "" + n / 50; },
        inverse: function (s) { return +s * 50; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,2}"
    },
    per_minute_to_per_second_0dp: {
        format: function (n) { return (n / 60).toFixed(0); },
        inverse: function (s) { return +s * 60; },
        regexp: regexp_util_1.NUMBER
    },
    per_minute_to_per_second_2dp: {
        format: function (n) { return (n / 60).toFixed(2); },
        inverse: function (s) { return +s * 60; },
        regexp: regexp_util_1.NUMBER + "\\.\\d{2}"
    },
    per_minute_to_per_second_2dp_if_required: {
        format: function (n) { return (n / 60).toFixed(2).replace(/\.?0*$/, ''); },
        inverse: function (s) { return +s * 60; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{0,2}"
    },
    milliseconds_to_seconds_0dp: {
        format: function (n) { return (n / 1000).toFixed(0); },
        inverse: function (s) { return +s * 1000; },
        regexp: regexp_util_1.NUMBER
    },
    milliseconds_to_seconds_2dp: {
        format: function (n) { return (n / 1000).toFixed(2); },
        inverse: function (s) { return +s * 1000; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d{2}"
    },
    multiplicative_damage_modifier: {
        format: function (n) { return "" + n; },
        inverse: function (s) { return +s; },
        regexp: regexp_util_1.NUMBER
    },
    '60%_of_value': {
        format: function (n) { return "" + n * 0.6; },
        inverse: function (s) { return +s / 0.6; },
        regexp: regexp_util_1.NUMBER + "\\.?\\d*"
    },
    id: {
        format: function (n) { return "" + n; },
        inverse: function (s) { return +s; },
        regexp: regexp_util_1.NUMBER
    },
    mod_value_to_item_class: mod_value_to_item_class_1.default,
    // TODO what is this
    canonical_stat: {
        format: function (n) { return "" + n; },
        inverse: function (s) { return +s; },
        regexp: regexp_util_1.NUMBER
    },
    placeholder: {
        format: function () { return '#'; },
        inverse: function () { return Number.NaN; },
        regexp: '#'
    }
};
function inverseFactory(formatter_id) {
    if (!exports.formatters.hasOwnProperty(formatter_id)) {
        throw new Error("'" + formatter_id + "' not found");
    }
    var inverse = exports.formatters[formatter_id].inverse;
    // TODO add ranges
    return inverse;
}
exports.inverseFactory = inverseFactory;
function regexpFactory(formatter_id) {
    if (!exports.formatters.hasOwnProperty(formatter_id)) {
        throw new Error("'" + formatter_id + "' not found");
    }
    var regexp = exports.formatters[formatter_id].regexp;
    // TODO add ranges
    return regexp;
}
exports.regexpFactory = regexpFactory;
function factory(formatter_id) {
    if (!exports.formatters.hasOwnProperty(formatter_id)) {
        throw new Error("'" + formatter_id + "' not found");
    }
    var format = exports.formatters[formatter_id].format;
    return function (value) {
        if (StatValue_1.isRange(value)) {
            if (value[0] === value[1]) {
                return String(format(value[0]));
            }
            else {
                var _a = __read(valueOrder(value, formatter_id), 2), min = _a[0], max = _a[1];
                return "(" + format(min) + " - " + format(max) + ")";
            }
        }
        else {
            return String(format(value));
        }
    };
}
exports.default = factory;
/**
 * orders the given values so that the smallest displayed is min
 *
 * reduced stats are given as negative values and then negated for display
 * whichs results in [-30, -15] being displayed as "(30 - 15) reduced"
 * @param param0
 *
 */
function valueOrder(_a, formatter_id) {
    var _b = __read(_a, 2), left = _b[0], right = _b[1];
    var sign = Math.sign(left);
    if ((left < right && sign === 1) || (left > right && sign === -1)) {
        return [left, right];
    }
    else {
        return [right, left];
    }
}
